// SLOS2 for Windows 3.1
// (c) 1994, 1998 Steven Lawrance

#include <windows.h>
#include "colorscr.h"

// SLOS2 Functions
int processSLOSCmd(char *cmdLine);
int processSLOSInstruction(char *cmdLine);
FILE *fopenSLOS(const char *name, const char *mode);
int execSLOS(FILE *stream);

// SLOS2 Variables
char slosFile[512];
char slosParams[512];
char slosString[512];
int slosNumber;

// SL-FAT Structure
typedef struct _SLOSFAT {
  char name[512];
  char file[512];
} SLOSFAT, *PSLOSFAT;
PSLOSFAT pFAT;
int nFiles;

void main(void) {
  // Ask for the program to run
  FILE *hFile = fopen("!ssl!.fat", "r");
  char cmdLine[256];

  // Initialize global variables
  pFAT = malloc(sizeof(SLOSFAT));
  nFiles = 0;

  // Initialize the classic color scheme
  textcolor(LIGHTGRAY);
  textbackground(BLUE);
  clrscr();

  // Print the SLOS greeting message
  printf("Welcome to SLOS2, a special edition of SLOS by Steven Lawrance\n\n");

  // Load the SL-FAT into memory
  if (hFile) {
	 while (fgets(pFAT[nFiles].name, 512, hFile) != NULL) {
		if (fgets(pFAT[nFiles].file, 512, hFile) == NULL) {
		  printf("SL-FAT has been corrupted!  Please run SLTOOLS.\n");
		  break;
		}

		// Increase the array size
		pFAT = realloc(pFAT, (++nFiles + 1) * sizeof(SLOSFAT));
		if (pFAT == NULL) {
		  printf("Not enough memory to load the SL-FAT!  Please free up room.\n");
		  nFiles = 0;
		}
	 }

	 // Close the FAT file
	 fclose(hFile);
  } else
	 printf("No SL-FAT was found!  When the SL-FAT is updated, one will be created.\n");

  // Run the "AutoExecutive File"
  hFile = fopenSLOS("AutoExecutive File", "r");
  if (hFile != NULL) {
	 // Run all the programs in this file
	 char progName[512];
	 FILE *hProgFile;
	 int nPrograms, i;
	 fscanf(hFile, "%i", &nPrograms);
	 for (i = 0; i < nPrograms; i++) {
		fgets(progName, 512, hFile);
		hProgFile = fopenSLOS(progName, "r");
		if (hProgFile != NULL) {
		  execSLOS(hProgFile);
		  fclose(hProgFile);
		} else
		  printf("Error 1: File \"%s\" not found\n", progName);
	 }
	 fclose(hFile);
  }

  // Run the menu loop
  do {
	 printf(":-) ");
	 gets(cmdLine);
  while (processSLOSCmd(cmdLine));
}

int processSLOSCmd(char *cmdLine) {
  // Process an SLOS command line
  char illegalValue = 0;
  int len = strlen(cmdLine);
  if (len > 0) {
	 switch (cmdLine[0]) {

		// Clear
		case 'C' : {
		  if (strncmp(cmdLine, "CLS", 3) == 0)
			 clrscr();
		  else
			 illegalValue = 1;
		  break;
		}

		// DIR,
		case 'D' : {
		  if (len > 1) {
			 switch (cmdLine[1]) {

				// DIR
				case 'I' : {
				  if (strncmp(cmdLine, "DIR", 3) == 0) {
					 // Write the SL-FAT in a directory
					 int i;
					 for (i = 0; i < nFiles; i++) {
						printf("%-12s %s\n", pFAT[i].file, pFAT[i].name);
						if (((i % 23) == 0) && (i != nFiles - 1)) {
						  printf("--More--");
						  getch();
						  gotoxy(1, wherey());
						  printf("        ");
						  gotoxy(1, wherey());
						}
					 }
				  }
				  break;
				}

				// DEL file
				case 'E' : {
				  if (strncmp(cmdLine, "DEL", 3) == 0) {
					 char fileName[256];
					 int slfatID;
					 if (strlen(cmdLine) > 4)) {
						// Get the parameter
						strcpy(fileName, (char *) (((long) cmdLine) + 4));
					 } else {
						// Ask for the file name
						printf("file: ");
						scanf("%s", fileName);
					 }

					 // Get the real file name
					 slfatID = getFileID(fileName);
					 if (slfatID >= 0) {
						// Delete the file
						if (unlink(fileName) == 0) {
						  // Remove the entry from the SL-FAT; file deleted
						  memcpy(pFAT[slfatID], pFAT[nFiles--], sizeof(SLOSFAT));
						} else
						  printf("File not found\n");
					 } else
						printf("File not found\n");
				  }
				  break;
				}

				// Default case
				default:
				  printf("Unrecognized command\n");
			 }
		  } else
			 printf("Unrecognized command\n");
		  break;
		}

		// FAT






int processSLOSInstruction(char *cmdLine) {
}

FILE *fopenSLOS(const char *name, const char *mode) {
  // Find the name in the SL-FAT
  char *file;
  int i = 0;
  char found = 0;
  while (!found && i < nFiles) {
	 if (strcmp(name, pFAT[i].name) == 0) {
		// File was found!
		file = pFAT[i].file;
		found = 1;
	 }
	 i++;
  }

  // If the file was found, then open it
  if (found)
	 return fopen(file, mode);
  else
	 return 0;
}

int execSLOS(FILE *stream) {
}


